/*
 * Copyright © 2015, 2016 Red Hat
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef GTK_PRIMARY_SELECTION_CLIENT_PROTOCOL_H
#define GTK_PRIMARY_SELECTION_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct gtk_primary_selection_device;
struct gtk_primary_selection_device_manager;
struct gtk_primary_selection_offer;
struct gtk_primary_selection_source;
struct wl_seat;

extern const struct wl_interface gtk_primary_selection_device_manager_interface;
extern const struct wl_interface gtk_primary_selection_device_interface;
extern const struct wl_interface gtk_primary_selection_offer_interface;
extern const struct wl_interface gtk_primary_selection_source_interface;

#define GTK_PRIMARY_SELECTION_DEVICE_MANAGER_CREATE_SOURCE	0
#define GTK_PRIMARY_SELECTION_DEVICE_MANAGER_GET_DEVICE	1
#define GTK_PRIMARY_SELECTION_DEVICE_MANAGER_DESTROY	2

#define GTK_PRIMARY_SELECTION_DEVICE_MANAGER_CREATE_SOURCE_SINCE_VERSION	1
#define GTK_PRIMARY_SELECTION_DEVICE_MANAGER_GET_DEVICE_SINCE_VERSION	1
#define GTK_PRIMARY_SELECTION_DEVICE_MANAGER_DESTROY_SINCE_VERSION	1

static inline void
gtk_primary_selection_device_manager_set_user_data(struct gtk_primary_selection_device_manager *gtk_primary_selection_device_manager, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) gtk_primary_selection_device_manager, user_data);
}

static inline void *
gtk_primary_selection_device_manager_get_user_data(struct gtk_primary_selection_device_manager *gtk_primary_selection_device_manager)
{
	return wl_proxy_get_user_data((struct wl_proxy *) gtk_primary_selection_device_manager);
}

static inline uint32_t
gtk_primary_selection_device_manager_get_version(struct gtk_primary_selection_device_manager *gtk_primary_selection_device_manager)
{
	return wl_proxy_get_version((struct wl_proxy *) gtk_primary_selection_device_manager);
}

static inline struct gtk_primary_selection_source *
gtk_primary_selection_device_manager_create_source(struct gtk_primary_selection_device_manager *gtk_primary_selection_device_manager)
{
	struct wl_proxy *id;

	id = wl_proxy_marshal_constructor((struct wl_proxy *) gtk_primary_selection_device_manager,
			 GTK_PRIMARY_SELECTION_DEVICE_MANAGER_CREATE_SOURCE, &gtk_primary_selection_source_interface, NULL);

	return (struct gtk_primary_selection_source *) id;
}

static inline struct gtk_primary_selection_device *
gtk_primary_selection_device_manager_get_device(struct gtk_primary_selection_device_manager *gtk_primary_selection_device_manager, struct wl_seat *seat)
{
	struct wl_proxy *id;

	id = wl_proxy_marshal_constructor((struct wl_proxy *) gtk_primary_selection_device_manager,
			 GTK_PRIMARY_SELECTION_DEVICE_MANAGER_GET_DEVICE, &gtk_primary_selection_device_interface, NULL, seat);

	return (struct gtk_primary_selection_device *) id;
}

static inline void
gtk_primary_selection_device_manager_destroy(struct gtk_primary_selection_device_manager *gtk_primary_selection_device_manager)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_primary_selection_device_manager,
			 GTK_PRIMARY_SELECTION_DEVICE_MANAGER_DESTROY);

	wl_proxy_destroy((struct wl_proxy *) gtk_primary_selection_device_manager);
}

struct gtk_primary_selection_device_listener {
	/**
	 * data_offer - introduce a new wp_primary_selection_offer
	 * @offer: (none)
	 *
	 * Introduces a new wp_primary_selection_offer object that may be
	 * used to receive the current primary selection. Immediately
	 * following this event, the new wp_primary_selection_offer object
	 * will send wp_primary_selection_offer.offer events to describe
	 * the offered mime types.
	 */
	void (*data_offer)(void *data,
			   struct gtk_primary_selection_device *gtk_primary_selection_device,
			   struct gtk_primary_selection_offer *offer);
	/**
	 * selection - advertise a new primary selection
	 * @id: (none)
	 *
	 * The wp_primary_selection_device.selection event is sent to
	 * notify the client of a new primary selection. This event is sent
	 * after the wp_primary_selection.data_offer event introducing this
	 * object, and after the offer has announced its mimetypes through
	 * wp_primary_selection_offer.offer.
	 *
	 * The data_offer is valid until a new offer or NULL is received or
	 * until the client loses keyboard focus. The client must destroy
	 * the previous selection data_offer, if any, upon receiving this
	 * event.
	 */
	void (*selection)(void *data,
			  struct gtk_primary_selection_device *gtk_primary_selection_device,
			  struct gtk_primary_selection_offer *id);
};

static inline int
gtk_primary_selection_device_add_listener(struct gtk_primary_selection_device *gtk_primary_selection_device,
					  const struct gtk_primary_selection_device_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) gtk_primary_selection_device,
				     (void (**)(void)) listener, data);
}

#define GTK_PRIMARY_SELECTION_DEVICE_SET_SELECTION	0
#define GTK_PRIMARY_SELECTION_DEVICE_DESTROY	1

#define GTK_PRIMARY_SELECTION_DEVICE_SET_SELECTION_SINCE_VERSION	1
#define GTK_PRIMARY_SELECTION_DEVICE_DESTROY_SINCE_VERSION	1

static inline void
gtk_primary_selection_device_set_user_data(struct gtk_primary_selection_device *gtk_primary_selection_device, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) gtk_primary_selection_device, user_data);
}

static inline void *
gtk_primary_selection_device_get_user_data(struct gtk_primary_selection_device *gtk_primary_selection_device)
{
	return wl_proxy_get_user_data((struct wl_proxy *) gtk_primary_selection_device);
}

static inline uint32_t
gtk_primary_selection_device_get_version(struct gtk_primary_selection_device *gtk_primary_selection_device)
{
	return wl_proxy_get_version((struct wl_proxy *) gtk_primary_selection_device);
}

static inline void
gtk_primary_selection_device_set_selection(struct gtk_primary_selection_device *gtk_primary_selection_device, struct gtk_primary_selection_source *source, uint32_t serial)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_primary_selection_device,
			 GTK_PRIMARY_SELECTION_DEVICE_SET_SELECTION, source, serial);
}

static inline void
gtk_primary_selection_device_destroy(struct gtk_primary_selection_device *gtk_primary_selection_device)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_primary_selection_device,
			 GTK_PRIMARY_SELECTION_DEVICE_DESTROY);

	wl_proxy_destroy((struct wl_proxy *) gtk_primary_selection_device);
}

/**
 * gtk_primary_selection_offer - offer to transfer primary selection
 *	contents
 * @offer: advertise offered mime type
 *
 * A wp_primary_selection_offer represents an offer to transfer the
 * contents of the primary selection clipboard to the client. Similar to
 * wl_data_offer, the offer also describes the mime types that the source
 * will transferthat the data can be converted to and provides the
 * mechanisms for transferring the data directly to the client.
 */
struct gtk_primary_selection_offer_listener {
	/**
	 * offer - advertise offered mime type
	 * @mime_type: (none)
	 *
	 * Sent immediately after creating announcing the
	 * wp_primary_selection_offer through
	 * wp_primary_selection_device.data_offer. One event is sent per
	 * offered mime type.
	 */
	void (*offer)(void *data,
		      struct gtk_primary_selection_offer *gtk_primary_selection_offer,
		      const char *mime_type);
};

static inline int
gtk_primary_selection_offer_add_listener(struct gtk_primary_selection_offer *gtk_primary_selection_offer,
					 const struct gtk_primary_selection_offer_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) gtk_primary_selection_offer,
				     (void (**)(void)) listener, data);
}

#define GTK_PRIMARY_SELECTION_OFFER_RECEIVE	0
#define GTK_PRIMARY_SELECTION_OFFER_DESTROY	1

#define GTK_PRIMARY_SELECTION_OFFER_RECEIVE_SINCE_VERSION	1
#define GTK_PRIMARY_SELECTION_OFFER_DESTROY_SINCE_VERSION	1

static inline void
gtk_primary_selection_offer_set_user_data(struct gtk_primary_selection_offer *gtk_primary_selection_offer, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) gtk_primary_selection_offer, user_data);
}

static inline void *
gtk_primary_selection_offer_get_user_data(struct gtk_primary_selection_offer *gtk_primary_selection_offer)
{
	return wl_proxy_get_user_data((struct wl_proxy *) gtk_primary_selection_offer);
}

static inline uint32_t
gtk_primary_selection_offer_get_version(struct gtk_primary_selection_offer *gtk_primary_selection_offer)
{
	return wl_proxy_get_version((struct wl_proxy *) gtk_primary_selection_offer);
}

static inline void
gtk_primary_selection_offer_receive(struct gtk_primary_selection_offer *gtk_primary_selection_offer, const char *mime_type, int32_t fd)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_primary_selection_offer,
			 GTK_PRIMARY_SELECTION_OFFER_RECEIVE, mime_type, fd);
}

static inline void
gtk_primary_selection_offer_destroy(struct gtk_primary_selection_offer *gtk_primary_selection_offer)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_primary_selection_offer,
			 GTK_PRIMARY_SELECTION_OFFER_DESTROY);

	wl_proxy_destroy((struct wl_proxy *) gtk_primary_selection_offer);
}

/**
 * gtk_primary_selection_source - offer to replace the contents of the
 *	primary selection
 * @send: send the primary selection contents
 * @cancelled: request for primary selection contents was canceled
 *
 * The source side of a wp_primary_selection_offer, it provides a way to
 * describe the offered data and respond to requests to transfer the
 * requested contents of the primary selection clipboard.
 */
struct gtk_primary_selection_source_listener {
	/**
	 * send - send the primary selection contents
	 * @mime_type: (none)
	 * @fd: (none)
	 *
	 * Request for the current primary selection contents from the
	 * client. Send the specified mime type over the passed file
	 * descriptor, then close it.
	 */
	void (*send)(void *data,
		     struct gtk_primary_selection_source *gtk_primary_selection_source,
		     const char *mime_type,
		     int32_t fd);
	/**
	 * cancelled - request for primary selection contents was
	 *	canceled
	 *
	 * This primary selection source is no longer valid. The client
	 * should clean up and destroy this primary selection source.
	 */
	void (*cancelled)(void *data,
			  struct gtk_primary_selection_source *gtk_primary_selection_source);
};

static inline int
gtk_primary_selection_source_add_listener(struct gtk_primary_selection_source *gtk_primary_selection_source,
					  const struct gtk_primary_selection_source_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) gtk_primary_selection_source,
				     (void (**)(void)) listener, data);
}

#define GTK_PRIMARY_SELECTION_SOURCE_OFFER	0
#define GTK_PRIMARY_SELECTION_SOURCE_DESTROY	1

#define GTK_PRIMARY_SELECTION_SOURCE_OFFER_SINCE_VERSION	1
#define GTK_PRIMARY_SELECTION_SOURCE_DESTROY_SINCE_VERSION	1

static inline void
gtk_primary_selection_source_set_user_data(struct gtk_primary_selection_source *gtk_primary_selection_source, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) gtk_primary_selection_source, user_data);
}

static inline void *
gtk_primary_selection_source_get_user_data(struct gtk_primary_selection_source *gtk_primary_selection_source)
{
	return wl_proxy_get_user_data((struct wl_proxy *) gtk_primary_selection_source);
}

static inline uint32_t
gtk_primary_selection_source_get_version(struct gtk_primary_selection_source *gtk_primary_selection_source)
{
	return wl_proxy_get_version((struct wl_proxy *) gtk_primary_selection_source);
}

static inline void
gtk_primary_selection_source_offer(struct gtk_primary_selection_source *gtk_primary_selection_source, const char *mime_type)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_primary_selection_source,
			 GTK_PRIMARY_SELECTION_SOURCE_OFFER, mime_type);
}

static inline void
gtk_primary_selection_source_destroy(struct gtk_primary_selection_source *gtk_primary_selection_source)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_primary_selection_source,
			 GTK_PRIMARY_SELECTION_SOURCE_DESTROY);

	wl_proxy_destroy((struct wl_proxy *) gtk_primary_selection_source);
}

#ifdef  __cplusplus
}
#endif

#endif
