#ifndef GTK_CLIENT_PROTOCOL_H
#define GTK_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct gtk_shell1;
struct gtk_surface1;
struct wl_surface;

extern const struct wl_interface gtk_shell1_interface;
extern const struct wl_interface gtk_surface1_interface;

#ifndef GTK_SHELL1_CAPABILITY_ENUM
#define GTK_SHELL1_CAPABILITY_ENUM
enum gtk_shell1_capability {
	GTK_SHELL1_CAPABILITY_GLOBAL_APP_MENU = 1,
	GTK_SHELL1_CAPABILITY_GLOBAL_MENU_BAR = 2,
	GTK_SHELL1_CAPABILITY_DESKTOP_ICONS = 3,
};
#endif /* GTK_SHELL1_CAPABILITY_ENUM */

/**
 * gtk_shell1 - gtk specific extensions
 * @capabilities: (none)
 *
 * gtk_shell is a protocol extension providing additional features for
 * clients implementing it.
 */
struct gtk_shell1_listener {
	/**
	 * capabilities - (none)
	 * @capabilities: (none)
	 */
	void (*capabilities)(void *data,
			     struct gtk_shell1 *gtk_shell1,
			     uint32_t capabilities);
};

static inline int
gtk_shell1_add_listener(struct gtk_shell1 *gtk_shell1,
			const struct gtk_shell1_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) gtk_shell1,
				     (void (**)(void)) listener, data);
}

#define GTK_SHELL1_GET_GTK_SURFACE	0
#define GTK_SHELL1_SET_STARTUP_ID	1
#define GTK_SHELL1_SYSTEM_BELL	2

#define GTK_SHELL1_GET_GTK_SURFACE_SINCE_VERSION	1
#define GTK_SHELL1_SET_STARTUP_ID_SINCE_VERSION	1
#define GTK_SHELL1_SYSTEM_BELL_SINCE_VERSION	1

static inline void
gtk_shell1_set_user_data(struct gtk_shell1 *gtk_shell1, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) gtk_shell1, user_data);
}

static inline void *
gtk_shell1_get_user_data(struct gtk_shell1 *gtk_shell1)
{
	return wl_proxy_get_user_data((struct wl_proxy *) gtk_shell1);
}

static inline uint32_t
gtk_shell1_get_version(struct gtk_shell1 *gtk_shell1)
{
	return wl_proxy_get_version((struct wl_proxy *) gtk_shell1);
}

static inline void
gtk_shell1_destroy(struct gtk_shell1 *gtk_shell1)
{
	wl_proxy_destroy((struct wl_proxy *) gtk_shell1);
}

static inline struct gtk_surface1 *
gtk_shell1_get_gtk_surface(struct gtk_shell1 *gtk_shell1, struct wl_surface *surface)
{
	struct wl_proxy *gtk_surface;

	gtk_surface = wl_proxy_marshal_constructor((struct wl_proxy *) gtk_shell1,
			 GTK_SHELL1_GET_GTK_SURFACE, &gtk_surface1_interface, NULL, surface);

	return (struct gtk_surface1 *) gtk_surface;
}

static inline void
gtk_shell1_set_startup_id(struct gtk_shell1 *gtk_shell1, const char *startup_id)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_shell1,
			 GTK_SHELL1_SET_STARTUP_ID, startup_id);
}

static inline void
gtk_shell1_system_bell(struct gtk_shell1 *gtk_shell1, struct gtk_surface1 *surface)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_shell1,
			 GTK_SHELL1_SYSTEM_BELL, surface);
}

#define GTK_SURFACE1_SET_DBUS_PROPERTIES	0
#define GTK_SURFACE1_SET_MODAL	1
#define GTK_SURFACE1_UNSET_MODAL	2
#define GTK_SURFACE1_PRESENT	3

#define GTK_SURFACE1_SET_DBUS_PROPERTIES_SINCE_VERSION	1
#define GTK_SURFACE1_SET_MODAL_SINCE_VERSION	1
#define GTK_SURFACE1_UNSET_MODAL_SINCE_VERSION	1
#define GTK_SURFACE1_PRESENT_SINCE_VERSION	1

static inline void
gtk_surface1_set_user_data(struct gtk_surface1 *gtk_surface1, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) gtk_surface1, user_data);
}

static inline void *
gtk_surface1_get_user_data(struct gtk_surface1 *gtk_surface1)
{
	return wl_proxy_get_user_data((struct wl_proxy *) gtk_surface1);
}

static inline uint32_t
gtk_surface1_get_version(struct gtk_surface1 *gtk_surface1)
{
	return wl_proxy_get_version((struct wl_proxy *) gtk_surface1);
}

static inline void
gtk_surface1_destroy(struct gtk_surface1 *gtk_surface1)
{
	wl_proxy_destroy((struct wl_proxy *) gtk_surface1);
}

static inline void
gtk_surface1_set_dbus_properties(struct gtk_surface1 *gtk_surface1, const char *application_id, const char *app_menu_path, const char *menubar_path, const char *window_object_path, const char *application_object_path, const char *unique_bus_name)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_surface1,
			 GTK_SURFACE1_SET_DBUS_PROPERTIES, application_id, app_menu_path, menubar_path, window_object_path, application_object_path, unique_bus_name);
}

static inline void
gtk_surface1_set_modal(struct gtk_surface1 *gtk_surface1)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_surface1,
			 GTK_SURFACE1_SET_MODAL);
}

static inline void
gtk_surface1_unset_modal(struct gtk_surface1 *gtk_surface1)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_surface1,
			 GTK_SURFACE1_UNSET_MODAL);
}

static inline void
gtk_surface1_present(struct gtk_surface1 *gtk_surface1, uint32_t time)
{
	wl_proxy_marshal((struct wl_proxy *) gtk_surface1,
			 GTK_SURFACE1_PRESENT, time);
}

#ifdef  __cplusplus
}
#endif

#endif
